@@ -1,5 +1,67 @@
 package Apache::Cache;
-#$Id: Cache.pm,v 1.6 2001/06/26 21:08:17 rs Exp $
+#$Id: Cache.pm,v 1.15 2001/08/29 07:45:32 rs Exp $
+
+=pod
+
+=head1 NAME
+
+Apache::Cache - Cache data accessible between Apache childrens
+
+=head1 SYNOPSIS
+
+    use Apache::Cache qw(:all);
+
+    my $cache = new Apache::Cache(cachename=>"dbcache", default_expires_in=>"5 minutes");
+
+    my $value = get_data('value_45');
+    $cache->set('value_45'=>$value);
+    print STDERR "can't save data in the cache" if($cache->status eq FAILURE);
+
+1 minute past
+
+    my $value = $cache->get('value_45');
+    # $value equal 'data'
+
+10 minutes past
+
+    my $value = $cache->get('value_45');
+    # $value equal 'undef()'
+    if($cache->status eq EXPIRED)
+    {
+        # update value
+        $cache->lock(LOCK_EX); # optional
+        $value = get_data('value_45');
+        $cache->set('value_45' => $value);
+        $cache->unlock;
+    }
+    elsif($cache->status eq FAILURE)
+    {
+        # don't use cache, cache maybe busy by another child
+        $value = get_data('value_45');
+    }
+
+=head1 DESCRIPTION
+
+This module allows you to cache data easily through shared memory. Whithin the framework 
+of an apache/mod_perl use, this cache is accessible from any child process. The data 
+validity is managed in the Cache::Cache model, but as well based on time than on size 
+or number of keys.
+
+=head1 USAGE
+
+For mod_perl users:
+
+in your httpd.conf, put this directive:
+
+    PerlAddVar PROJECT_DOCUMENT_ROOT /path/to/your/project/root/
+
+and in your startup.pl:
+
+    use Apache::Cache ();
+
+See L<Apache::SharedMem> for more details.
+
+=cut
 
 BEGIN
 {
@@ -23,9 +85,17 @@ BEGIN
     use constant EXPIRES_NOW    => 1;
     use constant EXPIRES_NEVER  => 0;
 
-    $Apache::Cache::VERSION     = '0.02';
+    $Apache::Cache::VERSION     = '0.04';
 }
 
+=pod
+
+=head1 METHODS
+
+=head2 new  (cachename=> 'cachename', default_expires_in=> '1 second', max_keys=> 50, max_size=> 1_000)
+
+=cut
+
 sub new
 {
     my $pkg     = shift;
@@ -158,7 +228,9 @@ sub set
         $self->_check_size($data);
 
         $self->SUPER::set($self->{cache_options}->{cachename}=>$data, NOWAIT);
-        return(undef()) if($self->status eq FAILURE);
+        my $rv = $self->status; # saving returned status
+        $self->unlock; # don't wait for Apache::SharedMem to auto unlock on destroy
+        return(undef()) if($rv eq FAILURE);
 
         return($value);
     }
@@ -180,14 +252,19 @@ sub get
     my($self, $key) = @_;
     
     my $data    = $self->_get_datas || return(undef());
-    my $timeout = $data->{_cache_metadata}->{timestamps}->{$key};
+    unless(exists $data->{$key})
+    {
+        $self->_set_status(EXPIRED);
+        return(undef());
+    }
     my $value   = $data->{$key};
+    my $timeout = $data->{_cache_metadata}->{timestamps}->{$key};
 
     if(!defined $timeout || ($timeout != EXPIRES_NEVER && $timeout <= time()))
     {
         $self->_set_error("data was expired");
+        $self->delete($key); # if delete failed, error string will be its own but not status
         $self->_set_status(EXPIRED);
-        $self->delete($key);
         return(undef());
     }
     else
@@ -243,16 +320,18 @@ sub _check_keys
             if($metadata->{timestamps}->{$metadata->{queue}->[$i]} > $time)
             {
                 my $key = $metadata->{queue}->[$i];
+                $self->_debug("$key is out of date, discarding");
                 delete($data->{$key});
                 delete($metadata->{timestamps}->{$key});
-                $metadata->{queue} = \@{grep($_ ne $key, @{$metadata->{queue}})};
+                @{$metadata->{queue}} = grep($_ ne $key, @{$metadata->{queue}});
                 last if(--$nkeys <= $nkeys_target);
             }
         }
-        if($nkeys > $max_keys)
+        if($nkeys > $nkeys_target)
         {
             # splice of delete candidates
-            my @key2del = splice(@{$metadata->{queue}}, 0, ($nkeys - $nkeys_target));
+            my @key2del = splice(@{$metadata->{queue}}, 0, ($nkeys - $nkeys_target - 1));
+            $self->_debug('cleaning not timed out keys: ', join(', ', @key2del));
             delete(@$data{@key2del});
             delete(@{$metadata->{timestamps}}{@key2del});
         }
@@ -285,3 +364,60 @@ sub _get_datas
 }
 
 1;
+
+=pod
+
+=head1 AUTHOR
+
+Olivier Poitrey E<lt>rs@rhapsodyk.netE<gt>
+
+=head1 LICENCE
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or (at
+your option) any later version.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with the program; if not, write to the Free Software
+Foundation, Inc. :
+
+59 Temple Place, Suite 330, Boston, MA 02111-1307
+
+=head1 COPYRIGHT
+
+Copyright (C) 2001 - Fininfo http://www.fininfo.fr
+
+=head1 PREREQUISITES
+
+Apache::Cache needs Apache::SharedMem available from the CPAN.
+
+=head1 SEE ALSO
+
+L<Apache::SharedMem>
+
+=head1 HISTORY
+
+$Log: Cache.pm,v $
+Revision 1.15  2001/08/29 07:45:32  rs
+add mod_perl specifique documentation
+
+Revision 1.14  2001/08/28 13:22:46  rs
+major bugfix: _check_keys method wasn't clean keys correctly
+
+Revision 1.13  2001/08/28 08:42:38  rs
+set method wasn't unlock on exit !
+
+Revision 1.12  2001/08/17 13:26:36  rs
+some minor pod modifications
+
+Revision 1.11  2001/08/17 13:20:45  rs
+- fix major bug in "get" method: on first timeout, status was set to
+  "delete" method's status (often SUCCESS) instead of EXPIRED
+- add some sections to pod documentation
+
